/**
 * \file plot.cpp
 * \date November 23, 2011
 * \author Edlira Nano
 */

#include "plot_aligned.h"
#include <qwt_legend.h>
#include <qwt_symbol.h>

PlotAligned::PlotAligned(QWidget *parent) :
		QwtPlot(parent) {

	setAutoReplot(false);

	setTitle("");

	// legend
	QwtLegend * legend = new QwtLegend;
	insertLegend(legend, QwtPlot::BottomLegend);

	setAxisTitle(QwtPlot::xBottom, "Retention time (s)");
	setAxisTitle(QwtPlot::yLeft, "Delta RT (s)");
	// enable zooming
	_zoomer = nullptr;
	
	_p_curve_deltart = new QwtPlotCurve("Aligned RT");
	_p_curve_ms2 = new QwtPlotCurve("Common identification");
}

PlotAligned::~PlotAligned() {
	this->clear();
	if (_zoomer != nullptr) {
		delete _zoomer;
		_zoomer = nullptr;
	}
}

void PlotAligned::viewAlignedMsRun(const Msrun * msrun) {
	qDebug() << "PlotAligned::viewAlignedMsRun begin";
	const std::vector<mcq_double> originalRt =
			msrun->getOriginalRetentionTimes();
	const std::vector<mcq_double> alignedRt = msrun->getAlignedRetentionTimes();
	if (originalRt.size() == alignedRt.size()) {
		std::vector<mcq_double>::const_iterator itOriginal = originalRt.begin();
		std::vector<mcq_double>::const_iterator itAligned = alignedRt.begin();
		//QwtPlotCurve * curve = new QwtPlotCurve("Aligned RT");
		_p_curve_deltart->setStyle(QwtPlotCurve::Lines);
		_p_curve_deltart->setRenderHint(QwtPlotItem::RenderAntialiased);
		_p_curve_deltart->setPen(QColor(Qt::blue));
		//_alignedPlots.push_back(curve);

		unsigned int plotsize(alignedRt.size());
		mcq_double x1[plotsize], y1[plotsize];
		int i = 0;
		for (; itOriginal != originalRt.end(); ++itOriginal, ++itAligned) {
			y1[i] = (*itOriginal) - (*itAligned);
			x1[i] = (*itOriginal);
			i++;
		}
		_p_curve_deltart->setSamples(x1, y1, plotsize);

		_p_curve_deltart->attach(this);

		this->setTitle(
				tr("Alignment of '%1'").arg(
						msrun->getXmlFileInfo().fileName()));
		this->replot();
		initZoomer();
	}
	
	qDebug() << "PlotAligned::viewAlignedMsRun end";
}

void PlotAligned::viewAlignedMS2Peaks(const MonitorAlignmentPlot * monitorAlignment) {
    qDebug() << "PlotAligned::viewAlignedMS2Peaks begin";
	if(monitorAlignment==0)
		return;

	qDebug() << "View Ms2 point";
    _p_curve_ms2->detach();
    delete (_p_curve_ms2);
    _p_curve_ms2 = nullptr;

    this->QwtPlot::replot();
	initZoomer();
    //delete _p_curve_ms2;
	_p_curve_ms2 = new QwtPlotCurve("Common identification");
    _p_curve_ms2->attach(this);
	//QwtPlotCurve * curve = new QwtPlotCurve("Common identification");
	_p_curve_ms2->setStyle(QwtPlotCurve::Dots);
	_p_curve_ms2->setRenderHint(QwtPlotItem::RenderAntialiased);
	_p_curve_ms2->setSymbol(new QwtSymbol(QwtSymbol::Star2, QColor(Qt::black), QColor(Qt::black),
			QSize(9, 9)));
	//_alignedPlots.push_back(curve);

	const std::map<mcq_double,mcq_double>& mapDeltaPeaks =	monitorAlignment->getMs2CommonPeak();
	
	
   //     QVector<QPointF> *data = static_cast<QVector<QPointF> *>(_p_curve_ms2->data());
    //_p_msms_data->clear();
	
	//_msms_data.clear();
    //_p_curve_ms2->detach();
    QVector< double > x;
    QVector< double > y;
    
	//unsigned int plotsize(mapDeltaPeaks.size());
	//mcq_double x1[plotsize], y1[plotsize];
	std::map<mcq_double,mcq_double>::const_iterator it;
	//int i = 0;
	for (it = mapDeltaPeaks.begin(); it != mapDeltaPeaks.end(); ++it) {
		QPointF one_point;
        x.push_back(it->first);
        y.push_back(it->second);
		//one_point.setX(it->first);
		//one_point.setY(it->second);
		//_msms_data.push_back(one_point);
		//y1[i] = it->second;
		//x1[i] =  it->first;
		//i++;
	}
	_p_msms_data = new QwtPointArrayData(x,y);
	_p_curve_ms2->setData(_p_msms_data);

	
	this->replot();
	initZoomer();
    qDebug() << "PlotAligned::viewAlignedMS2Peaks end";
}

void PlotAligned::clear() {
	qDebug() << "PlotAligned::clear() begin";
	/*
	if (_msms_data.size() > 0) {
		const QwtSymbol symbol(QwtSymbol::Star1, QColor(Qt::white), QColor(Qt::white),
			QSize(9, 9));
		_p_curve_ms2->setSymbol(symbol);
		//QwtData data(_p_curve_ms2->data());
		//QwtData * p_data = _p_curve_ms2->data().copy();
		_p_curve_ms2->setData(_msms_data);
		replot();
	}*/
	//_p_curve_ms2->setSamples(QVector<QPointF>());
	_p_curve_deltart->setSamples(QVector<QPointF>());
	_p_curve_ms2->detach();
	_p_curve_deltart->detach();
	

	setAxisAutoScale(QwtPlot::xBottom);
	setAxisAutoScale(QwtPlot::yLeft);
	this->setTitle("");
	//_msms_data.clear();
	replot();
	initZoomer();

	qDebug() << "PlotAligned::clear() end";
}

void PlotAligned::clearPlot() {
	this->clear();
}


void PlotAligned::initZoomer() {
	// LeftButton for the zooming
	// MidButton for the panning
	// RightButton: zoom out by 1
	// Ctrl+RighButton: zoom out to full size

	if (_zoomer != nullptr)
		delete _zoomer;

	_zoomer = new QwtPlotZoomer(canvas());
	_zoomer->setResizeMode(QwtPicker::KeepSize);
	_zoomer->setTrackerMode(QwtPicker::AlwaysOn);
	_zoomer->setMousePattern(QwtEventPattern::MouseSelect2, Qt::RightButton,
			Qt::ControlModifier);
	_zoomer->setMousePattern(QwtEventPattern::MouseSelect3, Qt::RightButton);

	const QColor c(Qt::darkBlue);
	_zoomer->setRubberBandPen(c);
	_zoomer->setTrackerPen(c);

}
