/**
 * \file lmatreader.cpp
 * \author Olivier Langella
 */

#include "lmatreader.h"
#include "../../saxparsers/xmlParserFactory.h"
#include "../mcq_error.h"
#include "../consoleout.h"

#include <iostream>
#include <QTime>

lmatReader::lmatReader() :
    LMat() {
}

lmatReader::~lmatReader() {
}

bool
lmatReader::set_from_xml(const Msrun & msrun, mcq_float mass_start,
                         mcq_float mass_end, mcq_float precision) {

    QFile file((msrun.getXmlFileInfo()).absoluteFilePath());
    qDebug() << "Obiwarp align, begin set from xml : " << msrun.getXmlFileInfo().absoluteFilePath() << endl;
    if (!file.open(QIODevice::ReadOnly)) {
        throw mcqError(QObject::tr("Unable to open input file '%1' :\n").arg(msrun.getXmlFileInfo().absoluteFilePath()));
    }
    // QTime tmzxml;
    // tmzxml.start();


    RawDataFormat raw_data_format = msrun.getRawDataFormat();
    XmlParserFactory xml_factory;
    XmlToLmatParser * xml_parser =
        xml_factory.newXmlToLmatParser(raw_data_format, this, mass_start, mass_end, precision);
    QXmlSimpleReader reader;
    QXmlInputSource xmlInputSource(&file);
    reader.setContentHandler(xml_parser);
    reader.setErrorHandler(xml_parser);

    if (reader.parse(xmlInputSource)) {
        file.close();
        mcqout() << "MS run '"
                 << (msrun.getXmlId())
                 << "' : xml file '"
                 << (msrun.getXmlFileInfo().absoluteFilePath())
                 << "' :  obiwarp parsing for lmat: OK" << endl;

        // qDebug("Obiwarp align set from xml parsing time was %d ms", tmzxml.elapsed());

    } else {
        file.close();
        throw mcqError(QObject::tr("error reading xml input file '%1' :\n").arg(msrun.getXmlFileInfo().absoluteFilePath()).append(xml_parser->errorString()));
        return (false);
    }

    delete (xml_parser);
    return (true);
}

