
/*******************************************************************************
 * Copyright (c) 2017 Olivier Langella <Olivier.Langella@u-psud.fr>.
 *
 * This file is part of the MassChroQ software.
 *
 *     MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <Olivier.Langella@u-psud.fr> - initial API and implementation
 ******************************************************************************/

/**
 * \file lib/peak_collections/alignedpeakcollectionondisk.cpp
 * \date January 06, 2017
 * \author Olivier Langella
 * \brief handles a collection of aligned peaks per MSruns on disk
 */

#include "alignedpeakcollectionondisk.h"

#include "../mass_chroq.h"

AlignedPeakCollectionOnDisk::AlignedPeakCollectionOnDisk()
{

}

AlignedPeakCollectionOnDisk::~AlignedPeakCollectionOnDisk()
{
    for (auto & element : _map_post_matched_aligned_peaks) {
        delete(element.second);
    }
}

void AlignedPeakCollectionOnDisk::add(const Msrun* p_current_msrun,AlignedXicPeakSp & xic_peak_sp) {
    qDebug() << "AlignedPeakCollectionOnDisk::add begin";

    if (_p_current_msrun != p_current_msrun) {
        AlignedPeakCollectionOnDisk::PeakCollectionFile * p_store_file = nullptr;
        std::pair<std::map<const Msrun*,PeakCollectionFile *>::iterator, bool> ret = _map_post_matched_aligned_peaks.insert(std::pair<const Msrun*, PeakCollectionFile *> (p_current_msrun,p_store_file));
        if (ret.second == true) {
            //OK new empty store file in map
            ret.first->second = new PeakCollectionFile();
            ret.first->second->_p_cachefile = new QTemporaryFile(MassChroq::tmp_dir.absolutePath() + "/" + QString("peak_collection_%1").arg(p_current_msrun->getXmlId()));
            //p_store_file->setAutoRemove(true);
            ret.first->second->_p_cachefile->open();
            ret.first->second->_p_outstream = new QDataStream(ret.first->second->_p_cachefile);
        }
        else {
            //store file already in map : may be a problem
            //throw mcqError("error in AlignedPeakCollectionOnDisk::add :\nthis should not happen : store file already in map : may be a problem");
        }
        _p_current_collection_file = ret.first->second;
        _p_current_msrun = p_current_msrun;
    }

    if (_p_current_collection_file == nullptr) {
        throw mcqError("error in AlignedPeakCollectionOnDisk::add :\n_p_current_collection_file == nullptr");
    }
    if (_p_current_collection_file->_p_outstream == nullptr) {
        throw mcqError("error in AlignedPeakCollectionOnDisk::add :\n_p_current_collection_file->_p_outstream == nullptr");
    }

    xic_peak_sp->serialize(_p_current_collection_file->_p_outstream);

    qDebug() << "AlignedPeakCollectionOnDisk::add end";
}


void AlignedPeakCollectionOnDisk::clear(const Msrun* p_current_msrun) {
    AlignedPeakCollectionOnDisk::PeakCollectionFile * p_store_file = _map_post_matched_aligned_peaks.at(p_current_msrun);
    p_store_file->close();
    delete p_store_file;
    _map_post_matched_aligned_peaks.erase(p_current_msrun);
}

std::vector<const Msrun*> AlignedPeakCollectionOnDisk::getMsRunList() {
    std::vector<const Msrun*> retval;
    for (auto const& element : _map_post_matched_aligned_peaks) {
        retval.push_back(element.first);
    }
    return retval;

}
std::vector<AlignedXicPeakSp> & AlignedPeakCollectionOnDisk::getMsRunAlignedPeakList(const Msrun* p_msrun) {
    qDebug() << "AlignedPeakCollectionOnDisk::getMsRunAlignedPeakList begin";
    _v_result.clear();
    auto it = _map_post_matched_aligned_peaks.find(p_msrun);
    if (it != _map_post_matched_aligned_peaks.end()) {

        AlignedPeakCollectionOnDisk::PeakCollectionFile * p_store_file = it->second;

        p_store_file->_p_cachefile->open();
        QDataStream instream(p_store_file->_p_cachefile);

        pappso::XicPeak peak;

        while (!instream.atEnd()) {
            AlignedXicPeak mcq_peak(peak, p_msrun);
            if (mcq_peak.unserialize(&instream)) {
                _v_result.push_back(std::make_shared<const AlignedXicPeak>(mcq_peak));
            }
        }
    }
    qDebug() << "AlignedPeakCollectionOnDisk::getMsRunAlignedPeakList end _v_result.size()=" << _v_result.size();
    return _v_result;
}

void AlignedPeakCollectionOnDisk::endMsrunQuantification(const Msrun* p_current_msrun) {
    qDebug() << "AlignedPeakCollectionOnDisk::endMsrunQuantification begin";

    auto it = _map_post_matched_aligned_peaks.find(p_current_msrun);
    if (it != _map_post_matched_aligned_peaks.end()) {

        it->second->close();
    }
    qDebug() << "AlignedPeakCollectionOnDisk::endMsrunQuantification end";

}
