#include "quantificationMethod.h"
#include "../mcq_error.h"
#include "../share/utilities.h"

QuantificationMethod::QuantificationMethod(const QString & xml_id)
    :
    _xml_id(xml_id)
{
}

QuantificationMethod::~QuantificationMethod()
{
qDebug() << "QuantificationMethod::~QuantificationMethod begin " ;
    delete _p_xic_extraction_method;
qDebug() << "QuantificationMethod::~QuantificationMethod 1 " ;
    delete _p_detection_method;
qDebug() << "QuantificationMethod::~QuantificationMethod 2 " ;
    // delete xic filters
    if (_p_xic_filter != nullptr ) {
        _p_xic_filter->deleteSink();
        delete _p_xic_filter;
    }
    
qDebug() << "QuantificationMethod::~QuantificationMethod end " ;
}

const QString &
QuantificationMethod::getXmlId() const
{
    return _xml_id;
}

void
QuantificationMethod::printInfos(QTextStream & out) const
{
    out << "\t Quantification method '" << _xml_id
        << "' parameters : " << endl;
    out << "\t xic type : '" << Utilities::toString(_xic_type) << "'" << endl;
    out << (_p_xic_extraction_method->printInfos());
    /*
    std::vector<const FilterBase *>::const_iterator it_f;
    for (it_f = _v_p_xic_filters.begin();
            it_f != _v_p_xic_filters.end();
            ++it_f) {
        out << ( (*it_f)->printInfos() );
    }
    */
    //out << (_p_detection_method->printInfos());
}

void
QuantificationMethod::setDetectionMethod(pappso::XicDetectionInterface * detection_method)
{
    _p_detection_method = detection_method;
}

const pappso::XicDetectionInterface *
QuantificationMethod::getDetectionMethod() const
{
    return _p_detection_method;
}

void
QuantificationMethod::setXicExtractionMethod(XicExtractionMethodBase * xic_extraction_method)
{
    _p_xic_extraction_method = xic_extraction_method;
}

const XicExtractionMethodBase *
QuantificationMethod::getXicExtractionMethod() const
{
    return _p_xic_extraction_method;
}

void
QuantificationMethod::setXicType(McqXicType xic_type)
{
    if ((xic_type == McqXicType::sum) || (xic_type == McqXicType::max))
    {
        _xic_type = xic_type;
    }
    else
    {
        throw mcqError(QObject::tr("error in QuantificationMethod::setXicType :\n _xic_type must be 'sum' or 'max' and not %1.\n").arg(Utilities::toString(xic_type)));
    }
}

McqXicType
QuantificationMethod::getXicType() const
{
    return _xic_type;
}

void
QuantificationMethod::addFilter(pappso::XicFilterInterface * p_filter)
{
    if (_p_xic_filter == nullptr) {
        _p_xic_filter = p_filter;
    }
    else {
        _p_xic_filter->getLastFilter()->setSink(p_filter);
    }
}

const pappso::XicFilterInterface *
QuantificationMethod::getXicFilters() const
{
    return _p_xic_filter;
}
