/*
    MassChroQ : Mass Chromatogram Quantification
    Copyright (C) 2014  Olivier Langella <Olivier.Langella@moulon.inra.fr>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "masschroq_cli.h"

#include <QDebug>



masschroqCli::masschroqCli(QObject *parent) :
    QObject(parent)
{
    // get the instance of the main application
    app = QCoreApplication::instance();
    // setup everything here
    // create any global objects
    // setup debug and warning mode
}



void
masschroqCli::windaube_exit()
{
//#if (WIN32)
//    mcqout() << "Press any key then enter to exit" << endl;
//    getchar();
//#endif
}

void
masschroqCli::display_help(QTextStream & out)
{
    out << "Usage: masschroq [OPTION] [FILE]" << endl;
    out << "Perform Mass Chromatogram Quantification as indicated in the XML input FILE." << endl;
    out << endl;
    out << "Options:" << endl;
    out << "  -h, --help\t\t\tdisplay this help and exit" << endl;
    out << endl;
    out << "  --validate\t\t\tcheck xml validity without running masschroq" << endl;
    out << endl;
    out << "  -t, --tmpdir DIRECTORY\tuse DIRECTORY as temporary working direcory for masschroq : temporary files generated by masschroq during execution time (one file or several slices at a time per xml data file being analysed) will go there. By default DIRECTORY is the current working directory of masschroq. Use this option if you want to set another one." << endl;
    out << endl;
    out << "  -p, --parse-peptides\t\tperform peptide text file parsing only, no quantification. If the identified peptides are given to masschroq via peptide text files (defined in FILE), he parses them, creates a new file called parsed-peptides_FILE which contains the FILE content (minus the lines relative to thr peptide text files to parse) plus the parsed peptides information. By default masschroq continues analysis and quantification on this new file. Use this option if you want masschroq only to parse the petide files but not to continue quantification on them. This allows you to check the parsed peptides and to add trace information on specific peptides before quantification." << endl;
    out << endl;
    out << "  -c, --cpus integer\tsets the number of cpus you want to use" << endl;
    out << endl;
    out << "  -v, --version\t\t\tdisplay version number and exit" << endl;
    //  out << "\n" << endl;
    out << endl;
    out << "For additional information, see the MassChroQ Homepage :" <<endl;
    out << "http://pappso.inra.fr/bioinfo/masschroq/" << endl;
    out.flush();
}



// 10ms after the application starts this method will run
// all QT messaging is running at this point so threads, signals and slots
// will all work as expected.
void masschroqCli::run()
{

    // QCoreApplication * app(this);
    // Add your main code here
    qDebug() << "MainClass.Run is executing";

    const QDateTime dt_begin = QDateTime::currentDateTime();
    const QString masschroq_dir_path(QCoreApplication::applicationDirPath());

    QStringList arguments = app->arguments();
    QString fileName;
    /// by default, temporary files go in the temporary directory of the system
    //  QString tmpDirName = QDir::tempPath();

    /// by default, temporary files go in the current working directory
    QString tmpDirName = QDir::currentPath();


    bool peptide_translation = false;
    bool masschroqml_validation = false;
    uint cpu_number = 1;

    int args_number = arguments.size();
    QString first, option;

    switch (args_number) {

    case 2 :
      qDebug() << "masschroqml cli case 2";
        first = arguments.at(1);
        if ( (first == "--help") ||
                (first == "-h") )
        {
            display_help(mcqout());
            windaube_exit();
            // you must call quit when complete or the program will stay in the
            // messaging loop
            quit();
        }
        else if ((first == "--version") ||
                 (first == "-v"))
        {
            mcqout() << QObject::tr("This is MassChroQ version %1,").arg(MASSCHROQ_VERSION) << endl;
            mcqout() << QObject::tr("using the masschroqML schema version %1.").arg(MASSCHROQ_SCHEMA_VERSION) << endl;
            windaube_exit();
// you must call quit when complete or the program will stay in the
            // messaging loop
            quit();
        }
        else
            fileName = first;
        break;

    case 3 :
      qDebug() << "masschroqml cli case 3";
        fileName = arguments.at(2);
        option = arguments.at(1);
        if ( (option == "--parse-peptides") ||
                (option == "-p"))
        {
            peptide_translation = true;
        }
        else if ((option == "--validate"))
        {
            masschroqml_validation = true;
        }

        else
        {
            mcqerr() << "Invalid use of option " << option << "." << endl;
            mcqerr() << "Try 'masschroq --help' for more information." << endl;
            windaube_exit();
            //return 1;
            app->exit(1);
        }
        break;
    case 4 :
      qDebug() << "masschroqml cli case 4";
        option = arguments.at(1);
        fileName = arguments.at(3);

        if ( (option == "--tmpdir") ||
                (option == "-t"))
        {
            tmpDirName = arguments.at(2);
        }
        else
        {
            if ( (option == "--cpus") ||
                    (option == "-c"))
            {
                cpu_number = arguments.at(2).toUInt();
                Quantificator::setCpuNumber(cpu_number);
            }
            else
            {
                mcqerr() << "Invalid use of option " << option << "." << endl;
                mcqerr() << "Try 'masschroq --help' for more information." << endl;
                windaube_exit();
                app->exit(1);
            }
        }
        break;

    case 6 :
        	  qDebug() << "masschroqml cli case 6";
       option = arguments.at(1);
        fileName = arguments.at(5);

        if ( (option == "--tmpdir") ||
                (option == "-t"))
        {
            tmpDirName = arguments.at(2);
        }
        else
        {
            if ( (option == "--cpus") ||
                    (option == "-c"))
            {
                cpu_number = arguments.at(2).toUInt();
                Quantificator::setCpuNumber(cpu_number);
            }
            else
            {
                mcqerr() << "Invalid use of option " << option << "." << endl;
                mcqerr() << "Try 'masschroq --help' for more information." << endl;
                windaube_exit();
                app->exit(1);
            }
        }
        option = arguments.at(3);
        if ( (option == "--tmpdir") ||
                (option == "-t"))
        {
            tmpDirName = arguments.at(4);
        }
        else
        {
            if ( (option == "--cpus") ||
                    (option == "-c"))
            {
                cpu_number = arguments.at(4).toUInt();
                Quantificator::setCpuNumber(cpu_number);
            }
            else
            {
                mcqerr() << "Invalid use of option " << option << "." << endl;
                mcqerr() << "Try 'masschroq --help' for more information." << endl;
                windaube_exit();
                app->exit(1);
            }
        }
        break;
    case 1 :
       	  qDebug() << "masschroqml cli case 1";
       mcqerr() << "masschroq : missing input filename." << endl;
        mcqerr() << "Try 'masschroq --help' for more information." << endl;
        windaube_exit();
        app->exit(1);
        break;
    default :
      	  qDebug() << "masschroqml cli default";

        mcqerr() << "masschroq : invalid use." << endl;
        mcqerr() << "Try 'masschroq --help' for more information." << endl;
        windaube_exit();
        app->exit(1);
    }

    //ReporterBase * logger = new ReporterConsole();
    //logger->write("Test HOOOOOO \n");

    MassChroq mass_chroq_engine;

    try
    {
        //defined application directory
        mass_chroq_engine.setMasschroqDir(masschroq_dir_path);
        mass_chroq_engine.setXmlFilename(fileName);

        if (masschroqml_validation == true) {
	  qDebug() << "masschroqml_validation == true";
            mass_chroq_engine.validateXmlFile();
// you must call quit when complete or the program will stay in the
            // messaging loop
           // quit();
        }
        else {
	  qDebug() << "masschroqml_validation == false";

            mass_chroq_engine.setTmpDir(tmpDirName);

            if (tmpDirName != QDir::tempPath())
            {
                mcqout() << "MassChroQ : temporary working directory is : "
                         << mass_chroq_engine.getTmpDirName() << endl;
            }
            mass_chroq_engine.setBeginDateTime(dt_begin);

            mass_chroq_engine.runDomParser();

            if (!peptide_translation) {
                mass_chroq_engine.runXmlFile();
            }

            const QDateTime dt_end = QDateTime::currentDateTime();

            const Duration dur =
                Utilities::getDurationFromDates(dt_begin, dt_end);

            mcqout() << "MassChroQ's execution time was : "
                     << Utilities::getDaysFromDuration(dur) << " days, "
                     << Utilities::getHoursFromDuration(dur) << " hours, "
                     << Utilities::getMinutesFromDuration(dur) << " minutes, "
                     << Utilities::getSecondsFromDuration(dur) << " seconds."
                     << endl;
        }
        windaube_exit();
// you must call quit when complete or the program will stay in the
        // messaging loop
        quit();
        //    delete logger;
    }
    catch (mcqError& error)
    {
        mcqerr() << "Oops! an error occurred in MassChroQ. Dont Panic :" << endl;
        mcqerr() << error.qwhat() << endl;

        windaube_exit();
	exit(1);
        app->exit(1);
    }
qDebug() << "before final quit()";

    // you must call quit when complete or the program will stay in the
    // messaging loop
    quit();
}

// call this routine to quit the application
void masschroqCli::quit()
{
    // you can do some cleanup here
    // then do emit finished to signal CoreApplication to quit
    emit finished();
}

// shortly after quit is called the CoreApplication will signal this routine
// this is a good place to delete any objects that were created in the
// constructor and/or to stop any threads
void masschroqCli::aboutToQuitApp()
{
    // stop threads
    // sleep(1);   // wait for threads to stop.
    // delete any objects
}

